part of '../bolter_provider.dart';

abstract class Presenter<FS> {
  late Bolter _bolter;
  late FS _flutterState;
  late BuildContext context;
  var _disposed = false;

  FS get flutterState => _flutterState;
  bool get disposed => _disposed;

  @protected
  ValueStream<V> stream<V>(V Function() getter, {bool distinct = true}) =>
      _bolter.stream(getter, distinct: distinct);

  @protected
  @Deprecated('use onLayout instead')
  FutureOr<void> init() {}

  @protected
  FutureOr<void> onLayout() {
    //todo: remove
    init();
  }

  @protected
  void initState() {}

  @protected
  void updateUI() => _bolter.shake();

  @protected
  Future<void> runAndUpdate<T>({
    void Function()? beforeAction,
    FutureOr<T> Function()? action,
    void Function()? afterAction,
    void Function(Object e)? onError,
  }) async {
    if (beforeAction != null) {
      beforeAction();
      _bolter.shake();
    }
    try {
      await action?.call();
      _bolter.shake();
    } catch (e) {
      if (onError != null) {
        onError(e);
        _bolter.shake();
      }
      rethrow;
    }
    if (afterAction != null) {
      afterAction();
      _bolter.shake();
    }
  }

  @protected
  @Deprecated(
      "handle context like in widgets, avoid calling context by throwing exceptions")
  void runContext(void Function(BuildContext context) handle) {
    handle(context);
  }

  @protected
  @mustCallSuper
  FutureOr<void> dispose() {
    _disposed = true;
  }
}

// mixin NavigationPresenter<FS> on Presenter<FS> {
//   ValueStream<Iterable<BolterRoute>> get routesStream => _bolter.stream(() => navigation.routes);
//
//   Navigation get navigation;
//
//   void pop() {
//     navigation.pop();
//     _bolter.shake();
//   }
//
//   void popUntil<T extends BolterRoute>() {
//     navigation.popUntil<T>();
//     _bolter.shake();
//   }
//
//   Future<R?> push<R>(BolterRoute<R?> route) {
//     navigation.push(route);
//     _bolter.shake();
//     return route.whenComplete;
//   }
//
//   Future<R?> pushReplacement<R>(BolterRoute<R?> route) {
//     navigation.pushReplacement(route);
//     _bolter.shake();
//     return route.whenComplete;
//   }
//
//   Future<R?> pushAndRemoveUntil<R>(BolterRoute<R?> route) {
//     navigation.pushAndRemoveUntil(route);
//     _bolter.shake();
//     return route.whenComplete;
//   }
//
//   List<BolterRoute> get currentRoutes => navigation.routes;
//
//   BolterRoute get currentRoute => currentRoutes.last;
//
//   T findRoute<T extends BolterRoute>() => navigation.findRoute<T>();
//
//   BolterRoute get firstRoute => currentRoutes.first;
//
//   bool isInStack<T extends BolterRoute>() => currentRoutes.whereType<T>().isNotEmpty;
//
//   bool get canPop => navigation.routes.length > 1;
//
//   BolterRoute get routeBefore => currentRoutes.length > 1
//       ? currentRoutes.elementAt(currentRoutes.length - 2)
//       : throw "No route";
// }
//
// mixin TabNavigationPresenter<FS> on Presenter<FS> {
//   TabNavigation get tabNavigation;
//
//   ValueStream<String> get currentTabStream => _bolter.stream(() => tabNavigation.tab);
//
//   String get currentTab => tabNavigation.tab;
//
//   void changeTab(String newTab) {
//     tabNavigation.tab = newTab;
//     _bolter.shake();
//   }
// }
